;NES Jack O' Lantern - Configurable Jack O' Lantern with Strobe Effect
;RG59 (C) 2015 Tyler N

iNesHeader  db  $4e,$45,$53,$1a,$01,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

;           =   $00
MODE        =   $01
TITLE       =   $02
VIEW        =   $03
MENU        =   $04
PPUFLAG     =   $05
OAMFLAG     =   $06
LOW         =   $07
HIGH        =   $08
PALNUM      =   $09
;           =   $0A
;           =   $0B
ROTPTR      =   $0C
TIMER       =   $0D
FRAME       =   $0E
MENUPTR     =   $0F
COLORS      =   $10
STATIC      =   $10
;           =   ...
ROTATING    =   $15
;           =   ...
DELAY       =   $1D
HIGHLIGHT   =   $1E
;           =   $1F
PALETTE     =   $20
;           =   ...
;           =   $3F
PAD_BUF     =   $40                         ;famitone2 pad poll
;           =   $41
;           =   $42
PAD_STATE   =   $43
PAD_STATEP  =   $44
PAD_STATET  =   $45


NTSC_MODE   =   $46                         ;famitone2
FT_PAUSE    =   $47
FT_TEMP     =   $48                         ;3 bytes in zeropage used as a scratchpad
;           =   $49
;           =   $4A


TRACK       =   $4B                         ;sound track
;           =   $4C
BUTTON      =   $4D                         ;konami code
CODEFLAG    =   $4E
CODEPTR     =   $4F
CODE        =   $50
;           =   $51
;           =   $52
;           =   $53
;           =   $54
;           =   $55
;           =   $56
;           =   $57
;           =   $58
;           =   $59


SPRITEDATA  =   $0200                       ;sprite data
SPRITEYPOS  =   $0200
SPRITETILE  =   $0201
SPRITEATTR  =   $0202
SPRITEXPOS  =   $0203


FT_BASE_ADR =   $0300                       ;page used for FT2 variables, should be $xx00
FT_DPCM_OFF =   $E000                       ;offset of dpcm data, $c000-$ffc0, 64-byte steps
FT_SFX_STREAMS  = 4                         ;number of sound effects played at once, 1..4
FT_DPCM_ENABLE                              ;undefine to exclude all DMC code
FT_SFX_ENABLE                               ;undefine to exclude all sound effects code
FT_THREAD                                   ;undefine if you are calling sound effects from the same thread as the sound update call
FT_PAL_SUPPORT                              ;undefine to exclude PAL support
FT_NTSC_SUPPORT                             ;undefine to exclude NTSC support


;-----------------------------------
    .org $C000                              ;prg rom 16k $C000-$FFFF
;-----------------------------------

Start       sei
            cld
            lda #%01000000                  ;APU inhibit irq
            sta $4017
            ldx #$ff
            txs                             ;stack pointer
            inx                             ;0
            stx $4010                       ;DMC disable irq
            stx $2000                       ;PPUCTRL disable nmi
            stx $2001                       ;PPUMASK disable rendering
            bit $2002                       ;clear VBlank flag
WaitVBlank1 bit $2002                       ;make sure 29658 cycles have passed
            bpl WaitVBlank1                 ;~27384 cyles have passed


            txa                             ;0
RAMLoop     sta $0000,x                     ;init
            sta $0100,x
            sta $0200,x
            sta $0300,x
            sta $0400,x
            sta $0500,x
            sta $0600,x
            sta $0700,x
            inx
            bne RAMLoop
            inx                             ;1
            stx VIEW
            stx TIMER
            stx TRACK
            inx                             ;2
            stx MENU
            jsr InitColors                  ;palette
            jsr InitNamTab                  ;nametable
            jsr InitSPD                     ;sprite data


WaitVBlank2 bit $2002
            bpl WaitVBlank2                 ;57165+ cyles have passed
            lda #%10000000                  ;turn on nmi to determine PAL or NTSC
            sta $2000
            lda FRAME
BlarggLoop  cmp FRAME                       ;wait NMI and VBlank
            beq BlarggLoop
            ldx #$34                        ;52
            ldy #$18                        ;24 (52 * 24 = 1,248)
NTSCLoop    dex
            bne NTSCLoop                    ;blargg's code from shiru's code
            dey                             ;taken from famitone2 code
            bne NTSCLoop
            lda $2002                       ;PPU_STATUS
            and #$80
            sta NTSC_MODE                   ;$00 = PAL, $80 = NTSC
            ldx #<trick_or_treat_music_data
            ldy #>trick_or_treat_music_data
            jsr FamiToneInit                ;init famitone2


            lda #%10010000                  ;nmi, 8x8, bg1, spr0, vram h, nt$2000
            sta $2000                       ;PPUCTRL
            sta OAMFLAG
            sta PPUFLAG

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

MainLoop    lda FRAME
WaitForRTI  cmp FRAME
            beq WaitForRTI

            ;...

            lda DELAY
            beq Expired                     ;no timer if delay is 0
            ldx TIMER
            beq ResetTimer                  ;if timer WAS expired, reset it
            dex
            jmp UpdateTimer                 ;tick timer
ResetTimer  tax
UpdateTimer stx TIMER                       ;if timer IS expired, rotate colors
            bne NotExpired
Expired     jsr RotatePal                   ;palette
            jsr FixPPU                      ;fix scroll
NotExpired  jsr FamiToneUpdate              ;update sound
            jsr PadPoll                     ;poll game pad
            jsr UserInput                   ;control the program
            jmp MainLoop


NMI         ldx #$00
            lda OAMFLAG
            beq NoOAM
            stx OAMFLAG
            lda #>SPRITEDATA
            sta $4014                       ;dma
NoOAM       lda PPUFLAG
            beq NoPPU
            stx PPUFLAG
            jsr SetPPU                      ;ppu
NoPPU       inc FRAME
Return      rti                             ;irq


FixPPU      lda #$00                        ;fix scroll
            sta $2005                       ;SCROLL_X
            sta $2005                       ;SCROLL_Y
SetPPU      lda #%10010000                  ;nmi, 8x8, bg1, spr0, vram h, nt$2000
            sta $2000                       ;PPUCTRL
            lda #%00011110                  ;show spr & bg, show left 8px sp & bg
            ldy MODE
            cpy VIEW
            beq ShowBG
            and #%00010110                  ;title and menu hide bg
ShowBG      sta $2001                       ;PPUMASK
            rts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

InitSPD     lda #$f8                        ;move sprites offscreen
SOSLoop     sta SPRITEDATA,x
            inx
            bne SOSLoop
            lda #$00                        ;init sprite data
            sta PALNUM                      ;title, view modes = palette 0
            lda #<SpriteMap
            sta LOW
            lda #>SpriteMap                 ;locate sprite map
            sta HIGH
            lda MODE
            beq BuildSPD                    ;title = 1st map
            cmp MENU
            bne SecondMap
            inc PALNUM                      ;menu = palette 1
ThirdMap    jsr NextMap                     ;menu = 3rd map
SecondMap   jsr NextMap                     ;view = 2nd map
            jmp BuildSPD                    ;build sprite data


NextMap     lda LOW
            clc
            adc #$e0                        ;each sprite map is $e0 bytes
            sta LOW                         ;16x14 224 possible positions
            bcc NoC
            inc HIGH
NoC         rts


BuildSPD    ldx #$00                        ;sprite data pointer
            ldy #$00                        ;x pos,y pos, sprite map pointer
BSLoop      lda (LOW),y
            cmp #$ff
            beq SkipBS                      ;)
            sta SPRITETILE,x                ;store sprite tile number
            lda PALNUM
            sta SPRITEATTR,x                ;store sprite attributes
            tya
            and #$0f                        ;get map x pos
            clc
            adc #$09                        ;offset +9 tiles
            asl
            asl
            asl                             ;mult 8
            sta SPRITEXPOS,x                ;store sprite x pos
            tya
            and #$f0                        ;get map y pos
            lsr
            lsr
            lsr
            lsr
            clc
            adc #$08                        ;offset +12 tiles
            asl
            asl
            asl                             ;mult 8
            sec
            sbc #$01                        ;match namtab y pos
            sta SPRITEYPOS,x                ;store sprite y pos
            inx
            inx
            inx
            inx                             ;next sprite
SkipBS      iny                             ;inc sprite map pointer
            cpy #$e0                        ;224 times
            bne BSLoop
            rts


InitNamTab  ldy #$00                        ;init name table
            ldx #$04
            lda #<NameTable
            sta LOW
            lda #>NameTable
            sta HIGH
            lda #$20
            sta $2006                       ;PPUADDR
            lda #$00
            sta $2006                       ;PPUADDR
INTLoop     lda (LOW),y
            sta $2007                       ;PPUDATA
            iny
            bne INTLoop
            inc HIGH
            dex
            bne INTLoop
            rts


PaletteMap  db  $00,$01,$02,$03,$00,$01,$02,$04,$00,$01,$02,$05,$00,$04,$05,$03
            db  $00,$03,$05,$04,$00,$0e,$0e,$0e,$00,$0f,$0f,$0f,$00,$0f,$0f,$0f
StatColors  db  $0f,$07,$06,$17,$28
RotColors   db  $28,$37,$38,$39,$30,$3a,$3b,$3c
Delay       db  $00
Highlight   db  $17,$30


InitColors  ldx #$0f
ICLoop      lda StatColors,x                ;copy from rom to ram
            sta COLORS,x
            dex
            bpl ICLoop
            ldx #$1f                        ;init palette
IPLoop      ldy PaletteMap,x
            lda StatColors,y                ;get pointer, look up color
            sta PALETTE,x                   ;store in ram
            dex
            bpl IPLoop
            jmp SetPalette                  ;will rts


RotatePal   ldy ROTPTR
            lda ROTATING,y                  ;get rotating color
            sta $2b
            sta $2e
            sta $32                         ;store in specific palette locations
            iny
            cpy #$08
            bne StoreRotPtr                 ;colors 0-7
            ldy #$00
StoreRotPtr sty ROTPTR


SetPalette  lda #$3f                        ;$3f00
            sta $2006                       ;PPUADDR
            ldy #$00
            sty $2006                       ;PPUADDR
SPLoop      lda PALETTE,y
            sta $2007                       ;PPUDATA
            iny
            cpy #$20                        ;all 32 colors
            bne SPLoop
            lda MODE                        ;get mode
            cmp MENU
            bne ExitPalette                 ;exit if not menu mode
            lda #$3f
            sta $2006
            lda #$00
            sta $2006
            lda #$0f                        ;overwrite bg with black
            sta $2007
ExitPalette rts


PadPoll     ldx #$00                        ;shiru's code famitone2
            jsr PadPollPort
            jsr PadPollPort
            jsr PadPollPort                 ;DPCM bug handling
            lda PAD_BUF
            cmp PAD_BUF+1
            beq PPDone
            cmp PAD_BUF+2
            beq PPDone
            lda PAD_BUF+1
PPDone      sta PAD_STATE
            lda PAD_STATE
            eor PAD_STATEP
            and PAD_STATE
            sta PAD_STATET
            lda PAD_STATE
            sta PAD_STATEP
            rts


PadPollPort ldy #$01
            sty $4016                       ;CTRL_PORT1
            dey
            sty $4016                       ;CTRL_PORT1
            ldy #$08
Poll1       lda $4016                       ;CTRL_PORT1
            and #$01
            clc
            beq Poll2
            sec
Poll2       ror PAD_BUF,x
            dey
            bne Poll1
            inx
            rts


MenuOffsets db  $38,$3c,$40,$44,$48
            db  $6c,$70,$74,$78,$7c,$80,$84,$88
            db  $a0


UserInput   lda MODE                        ;get mode: title, view, menu
            cmp MENU
            beq ChABut                      ;if menu mode check all buttons
            cmp TITLE
            bne ViewMode                    ;if view mode check only select & start
            jmp EnterCode                   ;if title check for code input
ViewMode    jmp ChSelectBut


ChABut      ldx MENUPTR
            ldy MenuOffsets,x               ;get offset
            lda PAD_STATET
            and #$01                        ;check if button A
            beq ChBBut
            sta SPRITEATTR,y                ;clear highlight
            inc MENUPTR                     ;inc pointer
            ldx MENUPTR
            cpx #$0e
            bcc NoMenuWrap
            ldx #$00                        ;forward wrap
            jmp MenuWrap


ChBBut      lda PAD_STATET
            and #$02                        ;check if button B
            beq ChPadU
            lda #$01
            sta SPRITEATTR,y                ;clear highlight
            dec MENUPTR                     ;dec pointer
            ldx MENUPTR
            bpl NoMenuWrap
            ldx #$0d                        ;reverse wrap


MenuWrap    stx MENUPTR                     ;store new pointer
NoMenuWrap  ldy MenuOffsets,x               ;get offset
            lda #$02
            sta SPRITEATTR,y                ;set highlight
            jmp SameData                    ;done with button a or b


ChPadU      lda PAD_STATET                  ;check pad up
            and #$10
            beq ChPadD
            lda SPRITETILE,y                ;get tile #
            clc
            adc #$10                        ;+1 chr row
            cmp #$40                        ;check for row $4
            bcc NoDWrap
            sec
            sbc #$40                        ;wrap to row $0, same column
NoUWrap     jmp StoreData


ChPadD      lda PAD_STATET                  ;check pad down
            and #$20
            beq ChPadL
            lda SPRITETILE,y                ;get tile #
            sec
            sbc #$10                        ;-1 chr row
            bpl NoUWrap                     ;check for row $f
            clc
            adc #$40                        ;wrap to row $3, same column
NoDWrap     jmp StoreData


ChPadL      lda PAD_STATET                  ;check pad left
            and #$40
            beq ChPadR
            ldx SPRITETILE,y                ;get tile #
            txa
            and #$0f                        ;check for column $0
            bne NoLWrap
            txa
            clc
            adc #$0d                        ;wrap to column $d, same row
            jmp LWrap
NoLWrap     dex
            txa                             ;-1 chr column
LWrap       jmp StoreData


ChPadR      lda PAD_STATET                  ;check pad right
            and #$80
            beq ChSelectBut
            ldx SPRITETILE,y                ;get tile #
            txa
            and #$0f
            cmp #$0d                        ;check for column $d
            bne NoRWrap
            txa
            and #$f0                        ;wrap to column $0, same row
            jmp RWrap
NoRWrap     inx
            txa                             ;+1 chr column
RWrap       jmp StoreData


ChSelectBut lda PAD_STATET
            and #$04                        ;check select button
            beq ChStartBut
            jmp MusicPlayer                 ;scroll through available songs


ChStartBut  lda PAD_STATET
            and #$08                        ;check start button
            beq EndUser
            jmp ChangeMode                  ;change mode


StoreData   sta SPRITETILE,y                ;set new tile #
            cmp #$0d                        ;IF TILE IS $0D DISGUISED AS $0F !!!!!
            bne NotOD
            lda #$0f                        ;LOAD $0F AS THE ACTUAL COLOR !!!!!!!!
NotOD       ldx MENUPTR
            sta COLORS,x                    ;store the data
            lda #$01
SameData    sta OAMFLAG                     ;dma later


EndUser     rts


EnterCode   lda #%00000001                  ;r,l,d,u,start,select,b,a
            sta BUTTON                      ;check buttons right to left
            jmp BeginCode
CodeLoop    asl BUTTON                      ;next button
            bcs ExitCode                    ;bit wrap = 8 buttons, exit
BeginCode   lda PAD_STATET
            cmp #%00000100                  ;skip if select button
            beq CodeLoop
            cmp #%00001000                  ;exit if start button
            beq CheckCode
            and BUTTON                      ;current button
            beq CodeLoop                    ;loop if this button is not pressed
            ldx CODEPTR
            cpx #$0a                        ;get code length
            bcs ExitCode                    ;exit if code is already 10 bytes
            sta CODE,x                      ;if less, store button as code byte
            inc CODEPTR                     ;move to next byte
            jmp CodeLoop                    ;...
ExitCode    rts


Key         db  $10,$10,$20,$20,$40,$80,$40,$80,$02,$01


CheckCode   ldx #$09                        ;check for match in reverse order
            jmp BeginCheck
CheckLoop   dex
            bmi KeyMatch                    ;if minus wrap, code matches key
BeginCheck  lda CODE,x
            cmp Key,x                       ;check code byte against key byte
            bne ChangeMode                  ;if bytes do not match exit check
            jmp CheckLoop                   ;if bytes do match check next byte
KeyMatch    inc CODEFLAG
            lda #$00                        ;reset
            sta CODEPTR
            jsr FamiToneMusicPlay           ;start hidden track


ChangeMode  ldx MODE                        ;get current mode
            cpx MENU                        ;title> view> <menu
            bne IncMode
            dex                             ;mode-=1
            jmp NewMode
IncMode     inx                             ;mode+=1
NewMode     stx MODE
            stx PPUFLAG
            stx OAMFLAG
            jsr InitSPD                     ;init sprite data
            lda MODE
            cmp MENU                        ;get new mode
            bne NotMenu                     ;skip if not menu
            ldx #$0d
ShowBytes   ldy MenuOffsets,x               ;get offset
            lda COLORS,x                    ;get saved data
            cmp #$0f                        ;IF COLOR IS $0F !!!!!!!!!!!!!!!!!!!!!!!
            bne SetTile
            lda #$0d                        ;SHOW TILE $0D DISGUISED AS $0F !!!!!!!!
SetTile     sta SPRITETILE,y                ;set tile #
            dex
            bpl ShowBytes
            ldx MENUPTR
            ldy MenuOffsets,x               ;get offset
            lda #$02
            sta SPRITEATTR,y                ;set highlight
NotMenu     ldy #$00
            sty ROTPTR                      ;reset rotate pointer
            iny
            sty TIMER                       ;reset timer
ChangePal   ldx #$1f                        ;all 32 colors
CPLoop      ldy PaletteMap,x                ;update palette
            lda COLORS,y
            sta PALETTE,x
            dex
            bpl CPLoop
EndChange   rts


MusicPlayer ldx TRACK                       ;cycle = [track two], track one, stop
            beq StopTrack                   ;0 stop playing track one
            dex
            beq TrackOne                    ;1 play track one
            jmp TrackTwo                    ;2 play track two
StopTrack   jsr FamiToneMusicStop
            ldx CODEFLAG                    ;1 if code was entered
            inx                             ;2 play track two next
            stx TRACK                       ;1 play track one next
            jmp End
TrackOne    ldx #<add_famwltz_music_data
            ldy #>add_famwltz_music_data
            dec TRACK                       ;0
            jmp PlayTrack
TrackTwo    ldx #<trick_or_treat_music_data
            ldy #>trick_or_treat_music_data
            dec TRACK                       ;1
PlayTrack   lda NTSC_MODE
            jsr FamiToneInit
            lda #$00
            jsr FamiToneMusicPlay
End         rts

;-----------------------------------

SpriteMap   incbin  "spritemap_title.bin"
            incbin  "spritemap_view.bin"
            incbin  "spritemap_menu.bin"


NameTable   incbin  "background.nam"


Famitone2   incsrc  "famitone2_asm6_alt.asm"
            incsrc  "trick_or_treat.asm"
            incsrc  "add_famwltz.asm"

;-----------------------------------
    .org $E000                              ;FT_DPCM_OFF
;-----------------------------------

            incbin  "trick_or_treat.dmc"


;-----------------------------------
    .org $FFFA                              ;vectors
;-----------------------------------

            dw  NMI
            dw  Start
            dw  Return                      ;irq


;-----------------------------------

Chr         incbin  "sprites.chr"
            incbin  "background.chr"